Package: PoissonBinomial
Type: Package
Title: Efficient Computation of Ordinary and Generalised Poisson
        Binomial Distributions
Version: 1.2.8
Date: 2026-03-06
Authors@R: person(
    "Florian", "Junge", 
    role = c("aut", "cre"), 
    email = "florian.junge@mailbox.org",
    comment = c(ORCID = "0009-0001-6856-6938")
  )
Maintainer: Florian Junge <florian.junge@mailbox.org>
Language: en-GB
Description: Efficient implementations of multiple exact and approximate methods
  as described in Hong (2013) <doi:10.1016/j.csda.2012.10.006>, Biscarri, Zhao &
  Brunner (2018) <doi:10.1016/j.csda.2018.01.007> and Zhang, Hong & Balakrishnan
  (2018) <doi:10.1080/00949655.2018.1440294> for computing the probability mass,
  cumulative distribution and quantile functions, as well as generating random
  numbers for both the ordinary and generalised Poisson binomial distribution.
License: GPL-3
Encoding: UTF-8
Imports: Rcpp (>= 1.0.11)
LinkingTo: Rcpp
SystemRequirements: fftw3 (>= 3.3)
Suggests: knitr, rmarkdown, microbenchmark
VignetteBuilder: knitr
URL: https://github.com/fj86/PoissonBinomial
BugReports: https://github.com/fj86/PoissonBinomial/issues
RoxygenNote: 7.3.3
NeedsCompilation: yes
Packaged: 2026-03-06 13:14:07 UTC; fjunge
Author: Florian Junge [aut, cre] (ORCID:
    <https://orcid.org/0009-0001-6856-6938>)
Repository: CRAN
Date/Publication: 2026-03-06 13:50:13 UTC
Built: R 4.6.0; x86_64-apple-darwin20; 2026-04-26 18:07:11 UTC; unix
Archs: PoissonBinomial.so.dSYM
