# Generated by roxygen2: do not edit by hand

export(dgpbinom)
export(dpbinom)
export(pgpbinom)
export(ppbinom)
export(qgpbinom)
export(qpbinom)
export(rgpbinom)
export(rpbinom)
import(Rcpp)
importFrom(stats,dbinom)
importFrom(stats,pbinom)
importFrom(stats,rbinom)
importFrom(stats,runif)
importFrom(stats,stepfun)
useDynLib(PoissonBinomial, .registration = TRUE)
