## Test environments
* local Windows 10 Education 22H2 10.0.19045.3693, R 4.3.2
* win-builder (configurations: "release", "oldrelease", "devel")
* R-hub (configurations: "sanitizers", "valgrind", "CRAN")


## R CMD check results

### local
0 errors | 0 warnings | 0 notes

### R-hub
* checking CRAN incoming feasibility ... [12s] NOTE
Maintainer: 'Florian Junge <florian.junge@mailbox.org>'

New maintainer:
  Florian Junge <florian.junge@mailbox.org>
Old maintainer(s):
  Florian Junge <florian.junge@h-da.de>
  
This change of addresses has been announced via eMail to
<cran-submissions@r-project.org> in advance.


### win-builder 
* checking CRAN incoming feasibility ... [12s] NOTE
Maintainer: 'Florian Junge <florian.junge@mailbox.org>'

New maintainer:
  Florian Junge <florian.junge@mailbox.org>
Old maintainer(s):
  Florian Junge <florian.junge@h-da.de>
  
This change of addresses has been announced via eMail to
<cran-submissions@r-project.org> in advance.